/* USER CODE BEGIN Header */
/**
  ******************************************************************************
  * @file           : main.c
  * @brief          : Main program body
  ******************************************************************************
  ** This notice applies to any and all portions of this file
  * that are not between comment pairs USER CODE BEGIN and
  * USER CODE END. Other portions of this file, whether 
  * inserted by the user or by software development tools
  * are owned by their respective copyright owners.
  *
  * COPYRIGHT(c) 2020 STMicroelectronics
  *
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met:
  *   1. Redistributions of source code must retain the above copyright notice,
  *      this list of conditions and the following disclaimer.
  *   2. Redistributions in binary form must reproduce the above copyright notice,
  *      this list of conditions and the following disclaimer in the documentation
  *      and/or other materials provided with the distribution.
  *   3. Neither the name of STMicroelectronics nor the names of its contributors
  *      may be used to endorse or promote products derived from this software
  *      without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
  * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
  ******************************************************************************
  */
/* USER CODE END Header */

/* Includes ------------------------------------------------------------------*/
#include "main.h"

/* Private includes ----------------------------------------------------------*/
/* USER CODE BEGIN Includes */
#include <string.h>
#include <stdbool.h>
#include "si5351.h"
/* USER CODE END Includes */

/* Private typedef -----------------------------------------------------------*/
/* USER CODE BEGIN PTD */

/* USER CODE END PTD */

/* Private define ------------------------------------------------------------*/
/* USER CODE BEGIN PD */

/* USER CODE END PD */

/* Private macro -------------------------------------------------------------*/
/* USER CODE BEGIN PM */

/* USER CODE END PM */

/* Private variables ---------------------------------------------------------*/
I2C_HandleTypeDef hi2c1;

TIM_HandleTypeDef htim1;

/* USER CODE BEGIN PV */
/* Private variables ---------------------------------------------------------*/

/* USER CODE END PV */

/* Private function prototypes -----------------------------------------------*/
void SystemClock_Config(void);
static void MX_GPIO_Init(void);
static void MX_TIM1_Init(void);
static void MX_I2C1_Init(void);
/* USER CODE BEGIN PFP */
/* Private function prototypes -----------------------------------------------*/

/* USER CODE END PFP */

/* Private user code ---------------------------------------------------------*/
/* USER CODE BEGIN 0 */

// note: can be affected a little by the actual load in the circuit
const int32_t XtalFilterCenterFrequency = 11997450;
int32_t Fvfo = 1000000; // dummy value, will be changed below

typedef enum {
	ROTARY_ENCODER_MODE_CHANGE_FREQEUNCY = 0,
	ROTARY_ENCODER_MODE_CHANGE_STEP = 1,
} RotaryEncoderMode_t;

int32_t prevCounter = 0;
int32_t lastFrequency40M = 0;
int32_t lastFrequency20M = 14060000;
uint8_t currentBand = 40;
int32_t targetFrequency = 7030000;
int32_t frequencyStep = 100;
int32_t prevFrequencyStep = 0;
RotaryEncoderMode_t rotaryEncoderMode;

#define PIN_RS    (1 << 0)
#define PIN_EN    (1 << 2)
#define BACKLIGHT (1 << 3)

#define LCD_DELAY_MS 5

uint8_t lcd_addr = (0x27 << 1);

#define BUTTON_ROT_ENCODER 0
#define BUTTON_DIT 1
#define BUTTON_DAH 2
bool inTransmitMode = false;
uint32_t transmitModeEnterTime = 0;

#define ROT_ENCODER_BUTTON_DEBOUNCE_TIME_MS 150
typedef enum {
	ROTARY_ENCODER_BUTTON_PRESSED = 0,
	ROTARY_ENCODER_BUTTON_RELEASED = 1,
	ROTARY_ENCODER_BUTTON_DEBOUNCE = 2,
} RotaryEncoderButtonStatus_t;

HAL_StatusTypeDef LCD_SendInternal(uint8_t data, uint8_t flags) {
    HAL_StatusTypeDef res;
    for(;;) {
        res = HAL_I2C_IsDeviceReady(&hi2c1, lcd_addr, 1,
                                    HAL_MAX_DELAY);
        if(res == HAL_OK)
            break;
    }

    uint8_t up = data & 0xF0;
    uint8_t lo = (data << 4) & 0xF0;

    uint8_t data_arr[4];
    data_arr[0] = up|flags|BACKLIGHT|PIN_EN;
    data_arr[1] = up|flags|BACKLIGHT;
    data_arr[2] = lo|flags|BACKLIGHT|PIN_EN;
    data_arr[3] = lo|flags|BACKLIGHT;

    res = HAL_I2C_Master_Transmit(&hi2c1, lcd_addr, data_arr,
                                  sizeof(data_arr), HAL_MAX_DELAY);
    HAL_Delay(LCD_DELAY_MS);
    return res;
}

void LCD_SendCommand(uint8_t cmd) {
    LCD_SendInternal(cmd, 0);
}

void LCD_SendData(uint8_t data) {
    LCD_SendInternal(data, PIN_RS);
}

void LCD_Init() {
    // 4-bit mode, 2 lines, 5x7 format
    LCD_SendCommand(0b00110000);
    // display & cursor home (keep this!)
    LCD_SendCommand(0b00000010);
    // display on, right shift, underline ON, blink off
    LCD_SendCommand(0b00001110);
    // clear display (optional here)
    LCD_SendCommand(0b00000001);
}

void LCD_SendString(char *str) {
    while(*str) {
        LCD_SendData((uint8_t)(*str));
        str++;
    }
}

void SetupCLK(uint8_t output, int32_t Fclk, si5351DriveStrength_t driveStrength) {
	static bool pllSetupDone = false;
	si5351PLLConfig_t pll_conf;
	si5351OutputConfig_t out_conf;

	si5351_Calc(Fclk, &pll_conf, &out_conf);
	if(!pllSetupDone) {
		si5351_SetupPLL(SI5351_PLL_A, &pll_conf);
		pllSetupDone = true;
	}
	si5351_SetupOutput(output, SI5351_PLL_A, driveStrength, &out_conf);
}

int32_t frequencyStepToDigitOffset(int32_t step) {
	// returns digit offset from the left depending on frequency step
	// example:
	// 14.000.00
	//      ^
	// step = 1000
	// offset = 5
	switch(step) {
		case 1:
			return 9;
		case 10:
			return 8;
		case 100:
			return 7;
		case 1000:
			return 5;
		case 10000:
			return 4;
		case 100000:
			return 3;
		default:
			return 0;
	}
}

void changeFrequencyStep(int32_t delta) {
	int32_t step = frequencyStep;
	while(delta != 0) {
		if(delta > 0) {
			step = step / 10;
			if(step < 100 ) {
				step = 10000;
			}
			delta--;
		} else {
			step = step * 10;
			if(step > 10000) {
				step = 100;
			}
			delta++;
		}
	}
	frequencyStep = step;
}

void changeFrequency(int32_t delta) {
	targetFrequency += frequencyStep*delta;

	if(currentBand == 40) {
		if(targetFrequency < 7000000) {
			targetFrequency = 7000000;
		} else if(targetFrequency > 7040000) {
			targetFrequency = 7040000;
		}
	} else { // 20m
		if(targetFrequency < 14000000) {
			targetFrequency = 14000000;
		} else if(targetFrequency > 14070000) {
			targetFrequency = 14070000;
		}
	}

	// use lowest VFO possible
	if(targetFrequency > XtalFilterCenterFrequency) {
		Fvfo = targetFrequency - XtalFilterCenterFrequency;
	} else {
		Fvfo = XtalFilterCenterFrequency - targetFrequency;
	}

	si5351_EnableOutputs((1<<2));
	SetupCLK(1, Fvfo, SI5351_DRIVE_STRENGTH_2MA);
	si5351_EnableOutputs((1 << 2)|(1<<1));
}

void changeRotaryEncoderMode() {
	if(rotaryEncoderMode == ROTARY_ENCODER_MODE_CHANGE_FREQEUNCY) {
		rotaryEncoderMode = ROTARY_ENCODER_MODE_CHANGE_STEP;
	} else {
		rotaryEncoderMode = ROTARY_ENCODER_MODE_CHANGE_FREQEUNCY;
	}
}

void displayCursor() {
	uint8_t digitOffset = frequencyStepToDigitOffset(frequencyStep);
    LCD_SendCommand(0b10000000 + digitOffset);
}

void displayFrequency() {
	char buff[16];
	snprintf(buff, sizeof(buff), "%02ld.%03ld.%01ld", 
		targetFrequency / 1000000, (targetFrequency % 1000000) / 1000, (targetFrequency % 1000) / 100);

    LCD_SendCommand(0b10000000);
    LCD_SendString(buff);

	displayCursor();
}

void enableTx12V() {
	HAL_GPIO_WritePin(GPIOB, GPIO_PIN_1, GPIO_PIN_SET);
}

void disableTx12V() {
	HAL_GPIO_WritePin(GPIOB, GPIO_PIN_1, GPIO_PIN_RESET);
}

void keyDown() {
	si5351_EnableOutputs((1 << 0)|(1 << 1)|(1 << 2));
	HAL_GPIO_WritePin(GPIOB, GPIO_PIN_0, GPIO_PIN_SET);
}

void keyUp() {
	si5351_EnableOutputs((1 << 1)|(1 << 2));
	HAL_GPIO_WritePin(GPIOB, GPIO_PIN_0, GPIO_PIN_RESET);
}

void switchRelayTx() {
	HAL_GPIO_WritePin(GPIOA, GPIO_PIN_0, GPIO_PIN_SET);
	HAL_Delay(10);
	HAL_GPIO_WritePin(GPIOA, GPIO_PIN_0, GPIO_PIN_RESET);
}

void switchRelayRx() {
	HAL_GPIO_WritePin(GPIOC, GPIO_PIN_15, GPIO_PIN_SET);
	HAL_Delay(10);
	HAL_GPIO_WritePin(GPIOC, GPIO_PIN_15, GPIO_PIN_RESET);
}

void switchRelay20M() {
	HAL_GPIO_WritePin(GPIOC, GPIO_PIN_14, GPIO_PIN_SET);
	HAL_Delay(10);
	HAL_GPIO_WritePin(GPIOC, GPIO_PIN_14, GPIO_PIN_RESET);
}

void switchRelay40M() {
	HAL_GPIO_WritePin(GPIOC, GPIO_PIN_13, GPIO_PIN_SET);
	HAL_Delay(10);
	HAL_GPIO_WritePin(GPIOC, GPIO_PIN_13, GPIO_PIN_RESET);
}

void changeBand() {
	if(currentBand == 40) {
		lastFrequency40M = targetFrequency;
		targetFrequency = lastFrequency20M;
		currentBand = 20;
		switchRelay20M();
	} else { // 20m
		lastFrequency20M = targetFrequency;
		targetFrequency = lastFrequency40M;
		currentBand = 40;
		switchRelay40M();
	}
	changeFrequency(0);
	displayFrequency();
}

void ensureTransmitMode() {
	if(inTransmitMode) {
		return;
	}

	// attenuate our own received signal by shifting VFO/BFO frequencies
	si5351_EnableOutputs(0);
	int32_t delta = 10000; // Hz
	if(currentBand == 40) {
		SetupCLK(1, Fvfo+delta, SI5351_DRIVE_STRENGTH_2MA);
	} else { // 20m
		SetupCLK(1, Fvfo-delta, SI5351_DRIVE_STRENGTH_2MA);
	}
	SetupCLK(2, XtalFilterCenterFrequency + 1000 + delta, SI5351_DRIVE_STRENGTH_2MA);
	si5351_EnableOutputs((1 << 2)|(1<<1));

	LCD_SendCommand(0b11000000);
	LCD_SendString("TX");
	displayCursor();

	// 2MA is an optimum drive level for given PA, don't try to change it
	SetupCLK(0, targetFrequency, SI5351_DRIVE_STRENGTH_2MA);
	// output is enabled/disabled only by keyDown()/keyUp() to prevent oscillations

	switchRelayTx();
	enableTx12V();

	inTransmitMode = true;
}

void ensureReceiveMode() {
	if(!inTransmitMode) {
		return;
	}

	LCD_SendCommand(0b11000000);
	LCD_SendString("RX");
	displayCursor();

	disableTx12V();
	switchRelayRx();

	// Restore original VFO and BFO
	si5351_EnableOutputs(0);
	SetupCLK(1, Fvfo, SI5351_DRIVE_STRENGTH_2MA);
	SetupCLK(2, XtalFilterCenterFrequency + 1000, SI5351_DRIVE_STRENGTH_2MA);

	si5351_EnableOutputs((1 << 2)|(1<<1)); // VFO, BFO

	inTransmitMode = false;
}

void init() {
/*
	// Inidcate that MCU is OK and firmware is running

	int i;
	for(i = 0; i < 7; i++) {
		// HAL_GPIO_WritePin(GPIOC, GPIO_PIN_13, GPIO_PIN_RESET);
		HAL_GPIO_TogglePin(GPIOC, GPIO_PIN_13);
		HAL_Delay(300);
	}
*/
/*
	// Code for determining the correction factor for Si5351

	si5351_Init(0);
	si5351PLLConfig_t pll_conf;
	si5351OutputConfig_t out_conf;
	int32_t Fclk = 10000000;

	si5351_Calc(Fclk, &pll_conf, &out_conf);
	si5351_SetupPLL(SI5351_PLL_A, &pll_conf);
	si5351_SetupOutput(0, SI5351_PLL_A, SI5351_DRIVE_STRENGTH_2MA, &out_conf);
	si5351_EnableOutputs(1<<0);

	while(1) {
		HAL_Delay(100);
	}
*/

	disableTx12V();
	keyUp();
	switchRelayRx();
	switchRelay40M();

	// Check for an alternative LCD address
    HAL_StatusTypeDef res;
    res = HAL_I2C_IsDeviceReady(&hi2c1, (0x3F << 1), 1, HAL_MAX_DELAY);
    if(res == HAL_OK) {
		lcd_addr = (0x3F << 1);
	}

	HAL_TIM_Encoder_Start(&htim1, TIM_CHANNEL_ALL);

    LCD_Init();
    LCD_SendCommand(0b10000000);
    LCD_SendString("  AYN1  ");
    LCD_SendCommand(0b11000000);
    LCD_SendString("Apr 2021");
	HAL_Delay(1000);
    // clear display
    LCD_SendCommand(0b00000001);

    LCD_SendCommand(0b10000000);
    LCD_SendString("I2C Scan:");
    LCD_SendCommand(0b11000000);
    for(uint16_t i = 0; i < 128; i++) {
		HAL_StatusTypeDef res;
        res = HAL_I2C_IsDeviceReady(&hi2c1, i << 1, 1, 10);
        if(res == HAL_OK) {
            char msg[64];
            snprintf(msg, sizeof(msg), "%02X ", i);
    		LCD_SendString(msg);
        }
    }  
	HAL_Delay(1000);
    // clear display
    LCD_SendCommand(0b00000001);

	LCD_SendCommand(0b11000000);
	LCD_SendString("RX");

	const int32_t correction = 6598;
	si5351_Init(correction);
	SetupCLK(2, XtalFilterCenterFrequency + 1000, SI5351_DRIVE_STRENGTH_2MA);
	changeFrequency(0);
  
    // display on, right shift, underline ON, blink off
    LCD_SendCommand(0b00001110);
	displayFrequency();
}

bool buttonRotEncoderLongPressed() {
	static uint32_t pressedSince = 0;
	uint32_t now = HAL_GetTick();
	if(HAL_GPIO_ReadPin(GPIOB, GPIO_PIN_10) == GPIO_PIN_RESET) {
		if(pressedSince == 0) {
			pressedSince = now;
		}
	} else {
		pressedSince = 0;
	}

	return (pressedSince > 0) && (now - pressedSince > 1000);
}

RotaryEncoderButtonStatus_t buttonRotEncoderPressed() {
	static uint32_t lastPressed = 0;
	if(HAL_GPIO_ReadPin(GPIOB, GPIO_PIN_10) == GPIO_PIN_RESET) {
		uint32_t now = HAL_GetTick();
		if(now - lastPressed > ROT_ENCODER_BUTTON_DEBOUNCE_TIME_MS) {
			lastPressed = now;
			return ROTARY_ENCODER_BUTTON_PRESSED;
		} else {
			return ROTARY_ENCODER_BUTTON_DEBOUNCE;
		}
	}
	return ROTARY_ENCODER_BUTTON_RELEASED;
}

bool buttonDitPressed() {
	return (HAL_GPIO_ReadPin(GPIOB, GPIO_PIN_11) == GPIO_PIN_RESET);
}

bool buttonDahPressed() {
	return (HAL_GPIO_ReadPin(GPIOB, GPIO_PIN_12) == GPIO_PIN_RESET);
}

int32_t sendStart = 0;
int32_t sendFinish = 0;
int32_t keyReadTimeout = 0;
int32_t sendTimeout = 0;
bool isSending = false;
bool straightKeyer = false;

typedef enum {
	CW_SEND_NONE = 0,
	CW_SEND_DIT = 1,
	CW_SEND_DAH = 2,
} CwSend_t;

CwSend_t lastSent = CW_SEND_NONE;
CwSend_t nextSend = CW_SEND_NONE;

// DIT_TIME_MS = 60*1000/(50*WPM)
// where 50 is the length in dits of "PARIS "
// see https://morsecode.world/international/timing.html
#define DIT_TIME_MS 65 // ~18-19 WPM

void processStraightKeyerLogic(bool pressed) {
	if(pressed) {
		keyDown();
	} else {
		keyUp();
	}
}

void processIambicKeyerLogic(bool ditPressed, bool dahPressed) {
	int32_t now = HAL_GetTick();

	if(isSending && (now >= sendFinish)) {
		keyUp();
		isSending = false;
	}

	if(nextSend == CW_SEND_NONE) {
		if(ditPressed && (lastSent == CW_SEND_DAH)) {
			nextSend = CW_SEND_DIT;
		} else if(dahPressed && (lastSent == CW_SEND_DIT)) {
			nextSend = CW_SEND_DAH;
		} else if(now > keyReadTimeout) {
			if(ditPressed) {
				nextSend = CW_SEND_DIT;
			} else if (dahPressed) {
				nextSend = CW_SEND_DAH;
			}
		}
	}

	if((now > sendTimeout) && (nextSend != CW_SEND_NONE)) {
		if(nextSend == CW_SEND_DIT) {
			sendFinish = now + DIT_TIME_MS;
			keyReadTimeout = now + (DIT_TIME_MS)*2;
		} else if(nextSend == CW_SEND_DAH) {
			sendFinish = now + (DIT_TIME_MS)*3;
			keyReadTimeout = now + (DIT_TIME_MS)*3;
		}

		lastSent = nextSend;
		nextSend = CW_SEND_NONE;
		sendStart = now;
		sendTimeout = sendFinish + DIT_TIME_MS;
		isSending = true;
		keyDown();
	}
}

void loop() {
	bool ditPressed = buttonDitPressed();
	bool dahPressed = buttonDahPressed();
	if(ditPressed || dahPressed) {
		transmitModeEnterTime = HAL_GetTick();
		ensureTransmitMode();
	} else {
		uint32_t tstamp = HAL_GetTick();
		if(tstamp - transmitModeEnterTime > (DIT_TIME_MS)*10) {
			ensureReceiveMode();
		}
	}

	int32_t currCounter = __HAL_TIM_GET_COUNTER(&htim1);
	currCounter = 32767 - ((currCounter-1) & 0xFFFF) / 2;
	if(currCounter > 32768/2) {
		// convert ... 32766, 32767, 0, 1, 2 ... into:
		//               ... -2, -1, 0, 1, 2 ...
		// this simplifies `delta` calculation
		currCounter = currCounter-32768;
	}
	if(currCounter != prevCounter) {
		int32_t delta = currCounter-prevCounter;
		prevCounter = currCounter;
		// debounce or skip counter overflow
		if((delta > -10) && (delta < 10) && (!inTransmitMode)) {
			delta = -delta; // the encoder was soldered backwards ¯\_(ツ)_/¯
			if(rotaryEncoderMode == ROTARY_ENCODER_MODE_CHANGE_FREQEUNCY) {
				changeFrequency(delta);
			} else {
				changeFrequencyStep(delta);
			}
			displayFrequency();
		}
	}

	if(inTransmitMode) {
		if(straightKeyer) {
			processStraightKeyerLogic(ditPressed);
		} else {
			processIambicKeyerLogic(ditPressed, dahPressed);
		}
	} else {
		if(buttonRotEncoderLongPressed()) {
			straightKeyer = !straightKeyer;

			LCD_SendCommand(0b11000000);
			LCD_SendString(straightKeyer? "STRAIGHT" : "IAMBIC  ");

			HAL_Delay(1000);

			LCD_SendCommand(0b11000000);
			LCD_SendString("RX      ");
			displayCursor();

			rotaryEncoderMode = ROTARY_ENCODER_MODE_CHANGE_FREQEUNCY;
		} else {
			static uint32_t lastStateChange = 0;
			static uint8_t changeStateTimes = 0;
			static RotaryEncoderButtonStatus_t prevStatus = ROTARY_ENCODER_BUTTON_RELEASED;
			RotaryEncoderButtonStatus_t status = buttonRotEncoderPressed();

			if(status == ROTARY_ENCODER_BUTTON_PRESSED) {
				changeRotaryEncoderMode();
			}

			if((status != prevStatus) && (status != ROTARY_ENCODER_BUTTON_DEBOUNCE)) {
				uint32_t now = HAL_GetTick();
				uint32_t delta = now - lastStateChange;
				if(delta < 500) { 
					changeStateTimes++;
				} else {
					changeStateTimes = 0;
				}
				lastStateChange = now;
				prevStatus = status;
			}

			if(changeStateTimes >= 3) { // double click
				changeBand();
				rotaryEncoderMode = ROTARY_ENCODER_MODE_CHANGE_FREQEUNCY;
				changeStateTimes = 0;
				lastStateChange = 0;
				prevStatus = ROTARY_ENCODER_BUTTON_RELEASED;
			}
		}
	}

    HAL_Delay(5);
}

/* USER CODE END 0 */

/**
  * @brief  The application entry point.
  * @retval int
  */
int main(void)
{
  /* USER CODE BEGIN 1 */

  /* USER CODE END 1 */

  /* MCU Configuration--------------------------------------------------------*/

  /* Reset of all peripherals, Initializes the Flash interface and the Systick. */
  HAL_Init();

  /* USER CODE BEGIN Init */

  /* USER CODE END Init */

  /* Configure the system clock */
  SystemClock_Config();

  /* USER CODE BEGIN SysInit */

  /* USER CODE END SysInit */

  /* Initialize all configured peripherals */
  MX_GPIO_Init();
  MX_TIM1_Init();
  MX_I2C1_Init();
  /* USER CODE BEGIN 2 */
  init();
  /* USER CODE END 2 */

  /* Infinite loop */
  /* USER CODE BEGIN WHILE */
  while (1)
  {
    loop();
    /* USER CODE END WHILE */

    /* USER CODE BEGIN 3 */

  }
  /* USER CODE END 3 */
}

/**
  * @brief System Clock Configuration
  * @retval None
  */
void SystemClock_Config(void)
{
  RCC_OscInitTypeDef RCC_OscInitStruct = {0};
  RCC_ClkInitTypeDef RCC_ClkInitStruct = {0};

  /**Initializes the CPU, AHB and APB busses clocks 
  */
  RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSI;
  RCC_OscInitStruct.HSIState = RCC_HSI_ON;
  RCC_OscInitStruct.HSICalibrationValue = RCC_HSICALIBRATION_DEFAULT;
  RCC_OscInitStruct.PLL.PLLState = RCC_PLL_NONE;
  if (HAL_RCC_OscConfig(&RCC_OscInitStruct) != HAL_OK)
  {
    Error_Handler();
  }
  /**Initializes the CPU, AHB and APB busses clocks 
  */
  RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_HCLK|RCC_CLOCKTYPE_SYSCLK
                              |RCC_CLOCKTYPE_PCLK1|RCC_CLOCKTYPE_PCLK2;
  RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_HSI;
  RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
  RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV1;
  RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV1;

  if (HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_0) != HAL_OK)
  {
    Error_Handler();
  }
}

/**
  * @brief I2C1 Initialization Function
  * @param None
  * @retval None
  */
static void MX_I2C1_Init(void)
{

  /* USER CODE BEGIN I2C1_Init 0 */

  /* USER CODE END I2C1_Init 0 */

  /* USER CODE BEGIN I2C1_Init 1 */

  /* USER CODE END I2C1_Init 1 */
  hi2c1.Instance = I2C1;
  hi2c1.Init.ClockSpeed = 100000;
  hi2c1.Init.DutyCycle = I2C_DUTYCYCLE_2;
  hi2c1.Init.OwnAddress1 = 0;
  hi2c1.Init.AddressingMode = I2C_ADDRESSINGMODE_7BIT;
  hi2c1.Init.DualAddressMode = I2C_DUALADDRESS_DISABLE;
  hi2c1.Init.OwnAddress2 = 0;
  hi2c1.Init.GeneralCallMode = I2C_GENERALCALL_DISABLE;
  hi2c1.Init.NoStretchMode = I2C_NOSTRETCH_DISABLE;
  if (HAL_I2C_Init(&hi2c1) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN I2C1_Init 2 */

  /* USER CODE END I2C1_Init 2 */

}

/**
  * @brief TIM1 Initialization Function
  * @param None
  * @retval None
  */
static void MX_TIM1_Init(void)
{

  /* USER CODE BEGIN TIM1_Init 0 */

  /* USER CODE END TIM1_Init 0 */

  TIM_Encoder_InitTypeDef sConfig = {0};
  TIM_MasterConfigTypeDef sMasterConfig = {0};

  /* USER CODE BEGIN TIM1_Init 1 */

  /* USER CODE END TIM1_Init 1 */
  htim1.Instance = TIM1;
  htim1.Init.Prescaler = 0;
  htim1.Init.CounterMode = TIM_COUNTERMODE_UP;
  htim1.Init.Period = 65535;
  htim1.Init.ClockDivision = TIM_CLOCKDIVISION_DIV1;
  htim1.Init.RepetitionCounter = 0;
  htim1.Init.AutoReloadPreload = TIM_AUTORELOAD_PRELOAD_DISABLE;
  sConfig.EncoderMode = TIM_ENCODERMODE_TI1;
  sConfig.IC1Polarity = TIM_ICPOLARITY_RISING;
  sConfig.IC1Selection = TIM_ICSELECTION_DIRECTTI;
  sConfig.IC1Prescaler = TIM_ICPSC_DIV1;
  sConfig.IC1Filter = 0;
  sConfig.IC2Polarity = TIM_ICPOLARITY_RISING;
  sConfig.IC2Selection = TIM_ICSELECTION_DIRECTTI;
  sConfig.IC2Prescaler = TIM_ICPSC_DIV1;
  sConfig.IC2Filter = 0;
  if (HAL_TIM_Encoder_Init(&htim1, &sConfig) != HAL_OK)
  {
    Error_Handler();
  }
  sMasterConfig.MasterOutputTrigger = TIM_TRGO_RESET;
  sMasterConfig.MasterSlaveMode = TIM_MASTERSLAVEMODE_DISABLE;
  if (HAL_TIMEx_MasterConfigSynchronization(&htim1, &sMasterConfig) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN TIM1_Init 2 */

  /* USER CODE END TIM1_Init 2 */

}

/**
  * @brief GPIO Initialization Function
  * @param None
  * @retval None
  */
static void MX_GPIO_Init(void)
{
  GPIO_InitTypeDef GPIO_InitStruct = {0};

  /* GPIO Ports Clock Enable */
  __HAL_RCC_GPIOC_CLK_ENABLE();
  __HAL_RCC_GPIOA_CLK_ENABLE();
  __HAL_RCC_GPIOB_CLK_ENABLE();

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(GPIOC, GPIO_PIN_13|GPIO_PIN_14|GPIO_PIN_15, GPIO_PIN_RESET);

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(GPIOA, GPIO_PIN_0, GPIO_PIN_RESET);

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(GPIOB, GPIO_PIN_0|GPIO_PIN_1, GPIO_PIN_RESET);

  /*Configure GPIO pins : PC13 PC14 PC15 */
  GPIO_InitStruct.Pin = GPIO_PIN_13|GPIO_PIN_14|GPIO_PIN_15;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(GPIOC, &GPIO_InitStruct);

  /*Configure GPIO pin : PA0 */
  GPIO_InitStruct.Pin = GPIO_PIN_0;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);

  /*Configure GPIO pins : PB0 PB1 */
  GPIO_InitStruct.Pin = GPIO_PIN_0|GPIO_PIN_1;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(GPIOB, &GPIO_InitStruct);

  /*Configure GPIO pins : PB10 PB11 PB12 */
  GPIO_InitStruct.Pin = GPIO_PIN_10|GPIO_PIN_11|GPIO_PIN_12;
  GPIO_InitStruct.Mode = GPIO_MODE_IT_RISING_FALLING;
  GPIO_InitStruct.Pull = GPIO_PULLUP;
  HAL_GPIO_Init(GPIOB, &GPIO_InitStruct);

  /* EXTI interrupt init*/
  HAL_NVIC_SetPriority(EXTI15_10_IRQn, 0, 0);
  HAL_NVIC_EnableIRQ(EXTI15_10_IRQn);

}

/* USER CODE BEGIN 4 */

/* USER CODE END 4 */

/**
  * @brief  This function is executed in case of error occurrence.
  * @retval None
  */
void Error_Handler(void)
{
  /* USER CODE BEGIN Error_Handler_Debug */
  /* User can add his own implementation to report the HAL error return state */
  while(1) 
  {
  }
  /* USER CODE END Error_Handler_Debug */
}

#ifdef  USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t *file, uint32_t line)
{ 
  /* USER CODE BEGIN 6 */
  /* USER CODE END 6 */
}
#endif /* USE_FULL_ASSERT */

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
